'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/** resolve selector */
function r$(amp, selector) {
    if (selector.includes(',')) {
        return selector.split(',').map(part => {
            if (/&/g.test(part)) {
                return part.replace(/&/g, amp);
            }
            else {
                return amp + ' ' + part;
            }
        }).join(', ');
    }
    else if (/&/g.test(selector)) {
        return selector.replace(/&/g, amp);
    }
    else {
        return amp + ' ' + selector;
    }
}
/** parse selector path */
function p$p(selectorPaths, instance) {
    let amp = '';
    selectorPaths.forEach(selector => {
        const adpatedSelector = typeof selector === 'string' ? selector : selector.$(instance.context);
        if (/,/g.test(amp)) {
            amp = amp
                .split(',')
                .map(ampPart => r$(ampPart, adpatedSelector))
                .join(', ');
        }
        else {
            amp = r$(amp, adpatedSelector);
        }
    });
    return amp.trim().replace(/\s+/g, ' ');
}

/** kebab regex */
const _kr = /[A-Z\u00C0-\u00D6\u00D8-\u00DE]/g;
/** kebab case */
function _kc(pattern) {
    return pattern.replace(_kr, match => '-' + match.toLowerCase());
}
/** _upwrap property */
/** TODO: refine it */
function _up(prop, indent = '  ') {
    if (typeof prop === 'object') {
        return (' {\n' +
            Object.entries(prop).map(v => {
                return indent + `  ${_kc(v[0])}: ${v[1]};`;
            }).join('\n') +
            '\n' + indent + '}');
    }
    return `: ${String(prop)};`;
}
/** unwrap properties */
function _ups(props, instance) {
    if (typeof props === 'function')
        return props(instance.context);
    return props;
}
/** create style */
function _cs(selector, props, instance) {
    if (props === null)
        return null;
    const unwrappedProps = _ups(props, instance);
    const propertyNames = Object.keys(unwrappedProps);
    if (propertyNames.length === 0) {
        if (instance.config.preserveEmptyBlock)
            return selector + ' {}';
        return null;
    }
    const statements = [
        selector + ' {'
    ];
    propertyNames.forEach(propertyName => {
        const property = unwrappedProps[propertyName];
        propertyName = _kc(propertyName);
        if (property !== undefined) {
            statements.push(`  ${propertyName}${_up(property)}`);
        }
    });
    statements.push('}');
    return statements.join('\n');
}
/** traverse */
function t(node, selectorPaths, styles, instance) {
    if (typeof node.$ === 'string') {
        selectorPaths.push(node.$);
    }
    else if (node.$.$ !== undefined) {
        if (node.$.before !== undefined)
            node.$.before(instance.context);
        selectorPaths.push(node.$.$(instance.context));
    }
    const selector = p$p(selectorPaths, instance);
    const style = _cs(selector, node.props, instance);
    if (style !== null)
        styles.push(style);
    if (node.children !== null) {
        node.children.forEach(childNode => {
            t(childNode, selectorPaths, styles, instance);
        });
    }
    selectorPaths.pop();
    if (!(typeof node.$ === 'string') && node.$.after !== undefined)
        node.$.after(instance.context);
}
function render(node, instance) {
    const styles = [];
    t(node, [], styles, instance);
    return styles.join('\n\n');
}

/* remove el */
function _re(el) {
    if (el === null)
        return;
    const parentElement = el.parentElement;
    if (parentElement !== null)
        parentElement.removeChild(el);
}
/** query element */
function _qe(id) {
    return document.querySelector(`style[css-render-id="${String(id)}"]`);
}
/** create element */
function _ce(id) {
    const el = document.createElement('style');
    el.setAttribute('css-render-id', String(id));
    return el;
}

function mount(instance, node, target) {
    let targetElement = null;
    if (target === undefined) {
        targetElement = document.createElement('style');
        document.head.appendChild(targetElement);
    }
    else if (typeof target === 'string' || typeof target === 'number') {
        targetElement = _qe(target);
        if (targetElement === null) {
            targetElement = _ce(target);
            document.head.appendChild(targetElement);
        }
        else {
            return targetElement;
        }
    }
    else {
        targetElement = target;
    }
    const style = node.render();
    if (targetElement.innerHTML !== style) {
        targetElement.innerHTML = style;
    }
    return targetElement;
}

/** render wrapper */
function _r() {
    return render(this, this.instance);
}
/** is node */
const _in = typeof document === 'undefined';
/** mount wrapper */
function _m(target) {
    if (_in)
        return null;
    const targetElement = mount(this.instance, this, target);
    const els = this.els;
    if (!els.includes(targetElement)) {
        els.push(targetElement);
    }
    return targetElement;
}
/** unmount */
function _um(target) {
    if (_in)
        return;
    const els = this.els;
    if (target === undefined) {
        els.forEach(_re);
        this.els = [];
    }
    else if (typeof target === 'string' || typeof target === 'number') {
        const targetElement = _qe(target);
        this.els = els.filter(el => el !== targetElement);
        _re(targetElement);
    }
    else {
        this.els = els.filter(el => el !== target);
        _re(target);
    }
}
/** traverse */
function _t(children, flattenedNodes) {
    children.forEach(child => {
        if (Array.isArray(child)) {
            _t(child, flattenedNodes);
        }
        else {
            flattenedNodes.push(child);
        }
    });
}
/** flatten */
function _f(children) {
    if (children === null)
        return null;
    const flattenedNodes = [];
    _t(children, flattenedNodes);
    return flattenedNodes;
}
/** create CNode */
function _cc(instance, $, props, children) {
    return {
        instance,
        $,
        props,
        children: _f(children),
        els: [],
        render: _r,
        mount: _m,
        unmount: _um
    };
}
const c = function (instance, $, props, children) {
    if (Array.isArray($)) {
        return _cc(instance, '', null, $);
    }
    if (Array.isArray(props)) {
        return _cc(instance, $, null, props);
    }
    else if (Array.isArray(children)) {
        return _cc(instance, $, props, children);
    }
    else {
        return _cc(instance, $, props, null);
    }
};

function CSSRender(config = {
    preserveEmptyBlock: false
}) {
    const cssr = {
        c: ((...args) => c(cssr, ...args)),
        use: (plugin, ...args) => plugin.install(cssr, ...args),
        id: Math.random().toString(36).slice(2, 10),
        context: {},
        config
    };
    return cssr;
}

exports.CSSRender = CSSRender;
exports.default = CSSRender;
